%model the phase-locking and level-locking properties of hair cell in inner
%ear.
%it generate phase-locking spike and ILD spikes according to a sound wave
%V7 is based on V6, it add ILD spike generation into the computation.
%V6 based on V5, it use centre frequency to speed up the computation.
%V5 based on V4, it generate the ITD_spike directly by comparing the index
%of the wave peak between two sound channels by skipping a period time on
%index.

%log on V7 revision
%1. add ILD spikes, move the condition of if
%last_amp(op_chan).value(temp_index)>ITD_thresh in V6 into the loop of if
%abs(ITD)<=Max_delay in order to add ILD code and avoid duplicating the
%time line sychronisation.
%2. add sr as input arguments


%log on V6 revision
%on 2008-06-09, 
%1. output the result into a ITD movie 
%2. find a situation that no ITD spike for some sound. see the google notes.
%(small binaural_delay, or too big ITD_thresh)
%3. add the index sychronise code to sychoronise the time line to detect the
%peak value. with the sychronising code, more ITD spike on high frequency
%channel are detected.
%on 2008-06-10
%1. replace the early finding peak stage with the find() function in
%matlab. It speeds up the process by 50%
%2. only use the max() function to detect the peak rather than using both
%min() and max(). it can save a little process time.


%2008-06-02
%author: jindong Liu

%input f_l: left sound in one specific frequency channel, f_r, right sound channel.
%MIND: THE f_l AND f_r should be 1*N array, i.e. ONE frequency channel.
%ITD_thresh, the amplitude threshhold when enabling phase-locking ITD cue.
% use it by comparing it with the independent amplitude of each channel.
%ILD_thresh: the amplitude threshold when enabling ILD cue. use this
%threshold by comparing it with the difference(in dB) of amplitude on two sound channels. 
%binaural_delay, the maximum delay time of sound arriving between two ears.
%in second.
%it is calculated by: (diameter of head)/soundspeed. 
%in near field, it has to be calculated by pi*diameter/soundspeed.
%fc: centre frequency of the input wave.

%output:
%ITD_spike: the spike train for each ITD level and frequency channel
%ITD_spike_sum: the spike sum for each ITD level and frequency channel
%ILD_spike
%ILD_spike_sum

%MIND: This code assume the max sound frequency is 20khz. assume both
%channel are non-zero.
%The ITD output is simulate the left side MSO, which sychronized with the
%left sound
%
function [ITD_spike,ITD_spike_sum, ILD_spike, ILD_spike_sum] = spikeGeneratorV7(f_l, f_r, ITD_thresh, ILD_thresh, binaural_delay, fc, sr)
%given a test sine wave


if nargin ~=7
    a_l = 0.1;      %amplitude
    a_r = 0.6;
    freq = 500;    %frequency
    sr = 44100; %sample rate
    fc = freq;
    t = 0:1/sr:20;
    binaural_delay = 0.25/344;  %=7.26e-4
    Max_delay = round(sr*binaural_delay);
    ITD_chan = Max_delay*2+1;  %1:Maxdelay+1 is mapping [-90 0]degree of sound. and [Max_delay+2, 2*Maxdelay+1] mapping (0,90) (-90 -left side sound, 90, right side sound)
    ILD_chan = 21;             %it covers [-ILD_thresh ILD_thresh], for example -3dB to 3dB amplitude difference
    
    f_l = a_l*sin(2*pi*freq*t);
    f_r = a_r*sin(2*pi*freq*(t-5.2e-4));
    sound = [f_l;f_r];         %combine for easy processing.
    ITD_thresh = 0.01;
    ILD_thresh =3;
    timeline = max(size(f_l));
    
    tic
else
    Max_delay = round(sr*binaural_delay);
    ITD_chan = Max_delay*2+1;  %1:Maxdelay+1 is mapping [-90 0]degree of sound. and [Max_delay+2, 2*Maxdelay+1] mapping (0,90) (-90 -left side sound, 90, right side sound)
    ILD_chan = 21;             %it covers [-ILD_thresh ILD_thresh], for example -3dB to 3dB amplitude difference
    sound = [f_l;f_r];         %combine for easy processing.

    timeline = max(size(f_l));
    t = 0:1/sr:(timeline-1)/sr; %sound duration.
    
end

%prepare spike train of the hair cell output for phase-locking &
%level-locking
ITD_spike = zeros(ITD_chan, timeline); %MIND: %the timing of spike always is sychronized to the left channel wave peaktime.
                                       %in another word, this is the left
                                       %MSO ITD model. you can do the same
                                       %thing on the right side if
                                       %necessary.
ILD_spike   =zeros(ILD_chan,timeline);
ILD_reso    = (2*ILD_thresh/(ILD_chan-1));  %the resolution of ILD_chan
ITD_spike_sum = zeros(ITD_chan,1);
ILD_spike_sum = zeros(ILD_chan,1);
period = round(sr/fc);

%build a structure to record the wave amplitude in previous wave period
%'value', record the amplitude value.
%'time', record the discrete time (actually the time line index of the
%wave) for the corresponding amplitude
%'now_i', the internal index for the latest amplitude, i.e.
%last_amp(1).value(last_amp(1).now_i) will return the most recent amplitude
%'size', the 'value' array size.
last_amp(1:2) = struct('value',zeros(1,Max_delay),'time',zeros(1,Max_delay),'now_i',0,'size',Max_delay);   
%build a struct for checking the amplitude in the present wave period.
now_amp(1:2)   =struct('value',0,'time',0);


%use find() function to localize the first position which is larger than
%ITD_thresh. This change can speed up the code by saving 50% time.
for chan=1:2
    pos = find(sound(chan,:)>ITD_thresh, 1);
    if ~isempty(pos)
        %find the first peak.
        [maxvalue,maxpos] = max(sound(chan,pos:pos+round(period/2)));
        now_amp(chan).time = pos+maxpos-1;
        now_amp(chan).value = maxvalue;
    else
        now_amp(chan).time = timeline;
        now_amp(chan).value = sound(chan,timeline);
    end
end

last_amp =pushamp(now_amp, last_amp,[1,2]);

for index = 1:2
    now_amp(index).value = 0;
    now_amp(index).time = 0;
end


j(1) = last_amp(1).time(last_amp(1).now_i);
j(2) = last_amp(2).time(last_amp(2).now_i);
while j(1) < (timeline-period*4) && j(2)<(timeline-period*4)
    %checking the amplitude of present period
    for chan=1:2 %1: left sound channel, 2: right sound channel.
        %temp_f = sound(chan,j);
        %find the minimal point
%         [min_val,min_pos] = min(sound(chan,j(chan):j(chan)+period));
%         %move the index to this minimal point
%         j(chan) = j(chan)+min_pos-1;
%         %find the maximal point
%         [max_val,max_pos] = max(sound(chan,j(chan):j(chan)+period));
%         %move the index to the maximal point
%         j(chan) = j(chan)+max_pos-1;
 
        %try only use max() to detect the maximal value. the running time
        %is similar to the method which uses both min and max. can speed up
        %a little but not much.
        [max_val,max_pos] = max(sound(chan,j(chan)+round(3*period/4):j(chan)+round(5*period/4)));
        j(chan) = j(chan)+round(3*period/4)+ max_pos-1;

        %update period to avoid frequency shift. %didn't see much
        %improvement.         
        %period = min_pos+max_pos-1;
        now_amp(chan).value = sound(chan,j(chan));
        now_amp(chan).time = j(chan);
        %last_amp = pushamp(now_amp, last_amp, chan);
        %replace the pushamp code with following code
        tmp_i = last_amp(chan).now_i;

        tmp_i = tmp_i+1;
        if tmp_i>Max_delay
            tmp_i=1;
        end
        last_amp(chan).value(tmp_i) = sound(chan,j(chan));
        last_amp(chan).time(tmp_i)  = j(chan);

        last_amp(chan).now_i = tmp_i;
        %end of replacement

        %to do, compare the last amp between two channels.
        switch chan
            case 1 %left channel get a new peak(new amplitude)
                op_chan = 2;  % the opposite channel is 2 now, i.e. right channel

            case 2
                op_chan = 1;
        end

        ITDtime  = last_amp(1).time(last_amp(1).now_i); %ITDtime always coordinate to the left channel.


        %the following code can be generalised for both side MSO
        %model to generate ITD spike although here it is used
        %only for left MSO.
        peaktime = last_amp(chan).time(last_amp(chan).now_i);
        new_amp = last_amp(chan).value(last_amp(chan).now_i);
        

        if new_amp>ITD_thresh %process ITD or ILD only if one of channel peak reach ITD_thresh.
            %compare the peaktime with the right channel
            %peak time and get the ITD and update the ITD
            %spike array
            %temp_index for the opposite channel.
            temp_index = last_amp(op_chan).now_i;
            
            while last_amp(op_chan).time(temp_index)~=0
                
                    ITD = peaktime - last_amp(op_chan).time(temp_index);
                    if abs(ITD)<=Max_delay
                        %ITD spike generation
                        if last_amp(op_chan).value(temp_index)>ITD_thresh
                            if chan ==1
                                ITD_spike(Max_delay+1+ITD,ITDtime) = 1;
                                ITD_spike_sum(Max_delay+1+ITD)=ITD_spike_sum(Max_delay+1+ITD)+1;
                            else
                                ITD_spike(Max_delay+1-ITD,ITDtime) = 1;
                                ITD_spike_sum(Max_delay+1-ITD)=ITD_spike_sum(Max_delay+1-ITD)+1;
                            end
                        end
                        %ILD spike generation
                        ILD = log(last_amp(op_chan).value(temp_index)/new_amp);
                        ILD = min(max(ILD, -ILD_thresh),ILD_thresh);   %limit the ILD in the range.
                        ILD_offset = round(ILD/ILD_reso);
                        
                        if chan ==1 %ILD<0 when sound from left.
                            ILD_spike((ILD_chan+1)/2+ILD_offset,ITDtime) = 1;
                            ILD_spike_sum((ILD_chan+1)/2+ILD_offset) = ILD_spike_sum((ILD_chan+1)/2+ILD_offset) +1;
                        else %ILD>0 when sound from right
                            
                            ILD_spike((ILD_chan+1)/2-ILD_offset,ITDtime) = 1;
                            ILD_spike_sum((ILD_chan+1)/2-ILD_offset) = ILD_spike_sum((ILD_chan+1)/2-ILD_offset) +1;

                        end
                            

                        
                    else
                        %synchronise the time line of two sound channel.
                        if temp_index ==last_amp(op_chan).now_i && abs(ITD)>period
                            j(chan) = j(chan)-fix(ITD/period)*period;
                            %disp('sychronise index on 2 channels.')
                        end
%                         disp(['rangout,j(',num2str(chan),')=',num2str(j(chan)), 'ITD=',num2str(ITD), 'Max_delay=',num2str(Max_delay),'\n']);
%                         disp(['peaktime=',num2str(peaktime),' op_chan.time=', num2str(last_amp(op_chan).time(temp_index))]);
%                         last_amp(:).time
                        break;
                    end
                
                temp_index = temp_index - 1;
                %decrese the temp_index to retrieve more
                %history last_amp.
                if temp_index<1
                    temp_index = Max_delay;
                end
                % check whether one cycle of comparing
                % finished.
                if temp_index == last_amp(op_chan).now_i
                    break;
                end
            end
        end




    end



end

%standalone display for testing this code independently.
if nargin ~=7
    toc
    figure('name','phase-locking');
    plot(t,f_l,'k',t,ITD_spike(10,:),'*');
    figure(2);
    plot(ITD_spike_sum);
    figure('name','Level-locking');
    plot(-ILD_thresh:ILD_reso:ILD_thresh, ILD_spike_sum);
    
end
    

end