
%by jindong liu
%18/12/2007
%to reconstrcut the spiking neural network model fro sound source
%localization
%paper source: A biologically inspired spiking neural network for sound
%source lateralization. Vol. 18, No. 6, Nov. 2007. by kyriakos Voutsas.

close all; clear all; clc

%sample step in second
%it is used in the differential function of the soma in mp.m

%load file via dialog
[wavfile, PathName] = uigetfile({'*.wav', 'All wave Files (*.wav)';...
            '*.*','All Files (*.*)'}, 'Load Wave File');

[pathstr,name,ext,versn] = fileparts(wavfile);
%cd(PathName) ;


[x,Fs,bits] = wavread([PathName wavfile]);

if size(x,1)>44100
    x = x(1:44100,:);
end
sr = Fs; % sampling rate

Gamm_total_chan = 16;
%time cost: 0.01s for 2336 input lines
[forward,feedback,fc,ERB,B] = GammaToneMake(sr,Gamm_total_chan,200,1200,'Moore'); %make filterbank

%change here to select the correct channel for each side.
y_left = GammaToneApply(x(:,1),forward,feedback); %filter into individual channels
y_right = GammaToneApply(x(:,2),forward,feedback); %filter into individual channels

figure('name','left'); subplot(1,2,1); BMMplot(y_left, fc, sr, [1 4 8 12 16]); %Label a selection of centre frequencies
subplot(1,2,2); BMMplot(y_left([1 4 8 12 16],:), fc, sr, []); % dont plot the centre frequencies on the ordinate and plot a selection of centre frequencies.

figure('name','right'); subplot(1,2,1); BMMplot(y_right, fc, sr, [1 4 8 12 16]); %Label a selection of centre frequencies
subplot(1,2,2); BMMplot(y_right([1 4 8 12 16],:), fc, sr, []); % dont plot the centre frequencies on the ordinate and plot a selection of centre frequencies.

%create time axis for future plot
timeAx = [1/sr:1/sr:size(y_left,2)*(1/sr)]*1e3;
%prepare label for later figure
freq_label = {};
ITD_label = {};
ILD_label = {};
for n=1:Gamm_total_chan
    freq_label(n) = cellstr(num2str(round(fc(n))));
end



figure('name',['Gammatonefilter',wavfile]);
plot(timeAx, y_left(5,:),timeAx,y_right(5,:));


% figure('name','left'); subplot(1,2,1); BMMplot(y_left, fc, sr, [1 4 8 12 16]); %Label a selection of centre frequencies
% subplot(1,2,2); BMMplot(y_left([1 4 8 12 16],:), fc, sr, []); % dont plot the centre frequencies on the ordinate and plot a selection of centre frequencies.
% 
% figure('name','right'); subplot(1,2,1); BMMplot(y_right, fc, sr, [1 4 8 12 16]); %Label a selection of centre frequencies
% subplot(1,2,2); BMMplot(y_right([1 4 8 12 16],:), fc, sr, []); % dont plot the centre frequencies on the ordinate and plot a selection of centre frequencies.
% 


ILD_spike_left = zeros(size(y_left));
ILD_spike_right= zeros(size(y_right));

binaural_delay = 8e-4;
Max_delay = round(sr*binaural_delay);
ITD_chan = Max_delay*2+1;  %1:Maxdelay+1 is mapping [-90 0]degree of sound. and [Max_delay+2, 2*Maxdelay+1] mapping (0,90) (-90 -left side sound, 90, right side sound)
ILD_chan = 21;

ITD_thresh = 0.001;
ILD_thresh = 3; %in dB

ITD_output = zeros(Gamm_total_chan,ITD_chan);
ITD_spike = cell(Gamm_total_chan,1);
ILD_output = zeros(Gamm_total_chan,ILD_chan);
ILD_spike = cell(Gamm_total_chan,1);

%prepare ITD label for figure
for n=1:ITD_chan
    str = sprintf('%3.2f',(n-(ITD_chan+1)/2)*1000/sr);
    ITD_label(n) = cellstr(str);
end
%prepare ILD label for figure
for n=1:ILD_chan
    ILD_label(n) = cellstr(num2str((n-(ILD_chan+1)/2)*(2*ILD_thresh/(ILD_chan-1))));
end

tic
for i_chan = 1: Gamm_total_chan
    f_l = y_left(i_chan,:);
    f_r = y_right(i_chan,:);
    [tmp_ITD_spike,tmp_ITD_spike_sum, tmp_ILD_spike, tmp_ILD_spike_sum] = spikeGeneratorV7(f_l, f_r, ITD_thresh, ILD_thresh, binaural_delay,fc(i_chan),sr);
    %[tmp_ITD_spike,tmp_ITD_spike_sum, tmp_ILD_spike, tmp_ILD_spike_sum] = spikeGeneratorV6_backup(f_l, f_r, ITD_thresh, ILD_thresh, binaural_delay,fc(i_chan));
    %[temp_phase_spike, temp_ILD_spike] = spikeGeneratorV4(f_l,f_r,0.01,0.01);
%    ITD_output(i_chan,:)=tmp_ITD_spike_sum;
    ITD_spike{i_chan} = tmp_ITD_spike;
    %spike number correction according to frequency.
    ITD_output(i_chan,:)=tmp_ITD_spike_sum.*fc(1)./fc(i_chan);

 %   ILD_output(i_chan,:) = tmp_ILD_spike_sum;
    ILD_spike{i_chan} = tmp_ILD_spike;
     %spike number correction according to frequency.
    ILD_output(i_chan,:)=tmp_ILD_spike_sum.*fc(1)./fc(i_chan);

end

disp('ITD time costing: ')
toc
figure('name','ITD distribution');
bar3(ITD_output,0.25,'detached');
title(['ITD only sound localisation result']);
set(gca, ...
    'ticklength'    , [0 0], ...
    'yticklabel'    , sprintf('%s|', freq_label{1:round(Gamm_total_chan/5):Gamm_total_chan}), ...
    'ylim'          , [0.5 Gamm_total_chan+0.5], ...
    'ytick'         , [1:round(Gamm_total_chan/5):Gamm_total_chan],...
    'xticklabel'    , sprintf('%s|', ITD_label{1:5:ITD_chan}), ...
    'xlim'          , [1 ITD_chan], ...
    'xtick'         , [1:5:ITD_chan]);
xlabel('L------sound source ITD (ms)------R');
ylabel('Freq(Hz)');
zlabel('Spiking number');


figure('name','ILD distribution');
bar3(ILD_output,0.25,'detached');
title(['ILD only sound localisation result']);
set(gca, ...
    'ticklength'    , [0 0], ...
    'yticklabel'    , sprintf('%s|', freq_label{1:round(Gamm_total_chan/5):Gamm_total_chan}), ...
    'ylim'          , [0.5 Gamm_total_chan+0.5], ...
    'ytick'         , [1:round(Gamm_total_chan/5):Gamm_total_chan],...
    'xticklabel'    , sprintf('%s|', ILD_label{1:3:ILD_chan}), ...
    'xlim'          , [1 ILD_chan], ...
    'xtick'         , [1:3:ILD_chan],...
    'PlotBoxAspectRatio',[4,1,1]);
xlabel('L------------sound source level difference----------------R');
ylabel('Centre freq (Hz)');
zlabel('Spiking');

%-----------------------------------------------------------------------
%record a movie to show the ITD distribution change according to time line.
%frame update every 0.01 second
figure('name',['ITD movie, file:', wavfile]);
r = subplot(6,1,1:4);
bar3(ITD_output,0.25,'detached');
%image(ITD_output);
title(['ITD only sound localisation result']);
set(gca, ...
    'ticklength'    , [0 0], ...
    'yticklabel'    , sprintf('%s|', freq_label{1:round(Gamm_total_chan/5):Gamm_total_chan}), ...
    'ylim'          , [0.5 Gamm_total_chan+0.5], ...
    'ytick'         , [1:round(Gamm_total_chan/5):Gamm_total_chan],...
    'xticklabel'    , sprintf('%s|', ITD_label{1:3:ITD_chan}), ...
    'xlim'          , [1 ITD_chan], ...
    'xtick'         , [1:3:ITD_chan]);
xlabel('L------------sound source ITD----------------R');
ylabel('Freq(Hz)');
zlabel('Spiking number');
axis tight
set(gca,'nextplot','replacechildren');
s = subplot(6,1,5);
plot(timeAx, x(:,1));
ylabel('L');
line([0,0],[-1,1]);
set(gca,'xticklabel',[]);
axis tight
set(gca,'nextplot','replacechildren');
t = subplot(6,1,6);
plot(timeAx, x(:,2));
ylabel('R');
line([0,0],[-1,1]);
axis tight
set(gca,'nextplot','replacechildren');
k = 1;
movstep = 0.01;
ITD_output = zeros(Gamm_total_chan,ITD_chan);
for i=movstep:movstep:(max(size(x))/sr)
    
    %[1 round(i*sr)]
    for j=1:Gamm_total_chan
        %[round((i-movstep)*sr+1) round(i*sr)]
        %ITD_output(j,:) =  sum(ITD_spike{j}(:,round((i-movstep)*sr+1):round(i*sr)),2)+ITD_output(j,:)';
        ITD_output(j,:) =  sum(ITD_spike{j}(:,round((i-movstep)*sr+1):round(i*sr)),2)*fc(1)/fc(j)+ITD_output(j,:)';
        %ITD_output(j,:) =  sum(ITD_spike{j}(:,1:round(i*sr)),2)';
    end
    axes(r);
    bar3(ITD_output,0.25,'detached');
    %image(ITD_output);
   	xlabel(['time now is ', num2str(i), 's']);
    axes(s);
    plot(timeAx, x(:,1));
    line([i*1000,i*1000],[-1,1],'color','k');
    
    axes(t);
    plot(timeAx, x(:,2));
    line([i*1000,i*1000],[-1,1],'color','k');
    F(k) = getframe(gcf);
    k = k+1;
    
end

% Play the movie ten times
% Play the movie; note that it does not fit the figure properly:
h2 = figure;
%movie(F,1)
% Use the figure handle to make the frames fit:
movie(h2,F,1);
%save movie
movie2avi(F,['ITDmov-',wavfile(1:end-4),'.avi'],'fps',15);

%----------------------------------------------------------------------
%record a movie to show the ILD distribution change according to time line.
%frame update every 0.01 second
figure('name',['ILD movie, file:', wavfile]);
r = subplot(6,1,1:4);
bar3(ILD_output,0.25,'detached');

title(['ILD only sound localisation result']);
set(gca, ...
    'ticklength'    , [0 0], ...
    'yticklabel'    , sprintf('%s|', freq_label{1:round(Gamm_total_chan/5):Gamm_total_chan}), ...
    'ylim'          , [0.5 Gamm_total_chan+0.5], ...
    'ytick'         , [1:round(Gamm_total_chan/5):Gamm_total_chan],...
    'xticklabel'    , sprintf('%s|', ILD_label{1:3:ILD_chan}), ...
    'xlim'          , [1 ILD_chan], ...
    'xtick'         , [1:3:ILD_chan],...
    'PlotBoxAspectRatio',[4,1,1]);
xlabel('L------------sound source level difference----------------R');
ylabel('Centre freq (Hz)');
zlabel('Spiking');

axis tight
set(gca,'nextplot','replacechildren');
s = subplot(6,1,5);
plot(timeAx, x(:,1));
ylabel('L');
set(gca,'xticklabel',[]);
line([0,0],[-1,1]);
axis tight
set(gca,'nextplot','replacechildren');

t = subplot(6,1,6);
plot(timeAx, x(:,2));
ylabel('R');
line([0,0],[-1,1]);
axis tight
set(gca,'nextplot','replacechildren');
k = 1;
movstep = 0.01;
ILD_output = zeros(Gamm_total_chan,ILD_chan);
for i=movstep:movstep:(max(size(x))/sr)
    
    %[1 round(i*sr)]
    for j=1:Gamm_total_chan
        %[round((i-movstep)*sr+1) round(i*sr)]
        %ILD_output(j,:) =  sum(ILD_spike{j}(:,round((i-movstep)*sr+1):round(i*sr)),2)+ILD_output(j,:)';
        ILD_output(j,:) =  sum(ILD_spike{j}(:,round((i-movstep)*sr+1):round(i*sr)),2)*fc(1)/fc(j)+ILD_output(j,:)';
        %ITD_output(j,:) =  sum(ITD_spike{j}(:,1:round(i*sr)),2)';
    end
    axes(r);
    bar3(ILD_output,0.25,'detached');
    %image(ITD_output);
   	xlabel(['time now is ', num2str(i), 's']);
    set(gca,'PlotBoxAspectRatio',[4,1,1])
    axes(s);
    plot(timeAx, x(:,1));
    line([i*1000,i*1000],[-1,1],'color','k');
    
    axes(t);
    plot(timeAx, x(:,2));
    line([i*1000,i*1000],[-1,1],'color','k');
    F(k) = getframe(gcf);
    k = k+1;
    
end

% Play the movie ten times
% Play the movie; note that it does not fit the figure properly:
h2 = figure;
%movie(F,1)
% Use the figure handle to make the frames fit:
movie(h2,F,1);
%save movie
movie2avi(F,['ILDmov-',wavfile(1:end-4),'.avi'],'fps',15);

